packCircles <- function(x, sizecol=NULL, sizetype = c("area","radius"), 
                       groupcol=NULL, labcol=NULL,
                       groupsort=FALSE, decreasing = FALSE, 
                       pal=blues9[3:9], col.line=NULL, lwd=2, 
                       col.lab=NULL, col.lab.defaults=c(par("bg"),par("fg")), 
                       cex.lab = par("cex.lab"),
                       main = NULL, sub = NULL,
                       plot=TRUE, annot=TRUE, add=FALSE, ...){
## function uses circle packing algorithm 
## implemented in package packcircles
  
  ## first round of input checks
  stopifnot(is.numeric(x) || is.character(x) || is.factor(x) || 
              is.matrix(x) || is.data.frame(x))
  stopifnot(length(col.lab.defaults)==2)
  areColors <- function(x) {
    tryCatch(is.matrix(vapply(x, col2rgb, c(0,0,0))), 
             error = function(e) FALSE)
  }
  stopifnot(all(areColors(pal), areColors(col.lab), 
                areColors(col.line), areColors((col.lab.defaults))))
  stopifnot(is.logical(groupsort), is.logical(decreasing))
  stopifnot(is.numeric(cex.lab), is.numeric(lwd))
  stopifnot(cex.lab>0, lwd>0)
  stopifnot(all(sizetype %in% c("area", "radius")))
  
  xnam <- deparse(substitute(x))
  if (!is.matrix(x) && !is.data.frame(x)) {
    ## vector valued x
    ## a factor, or a character vector, or a numeric vector
       xord <- x
       n <- length(xord)
         if (is.factor(xord) || is.character(xord)){
           ## categorical
             xord=table(xord)
             namvec <- c(xnam, "count")
             xord <- data.frame(xord)
             colnames(xord) <- namvec
             sizecol <- "count"
             xord$label <- paste(xord[,xnam], paste0(xord[,"count"], " (",
                          round(100*xord[,"count"]/sum(xord[,"count"]),1), "%)"), sep="\n")
             labcol="label"
             grp <- rep(1, length(xord))
             groupcol <- NULL
         }
         else {
           ## quantitative x
           rn <- names(xord)
              ## names of a named vector, remains NULL otherwise
           if (!any(length(sizecol)==n, 
                    length(groupcol)==n, 
                    length(labcol)==n)){
                       ## no valid accompanying single columns are provided
                       if(!is.null(sizecol)) warning("sizecol ignored, assumed to be", xnam)
                       if(!all(is.null(groupcol), is.null(labcol)))
                         warning("invalid groupcol or labcol was specified (wrong length)")
                       sizecol <- xnam
                       groupcol <- NULL
                       xord <- data.frame(xord)
                       namvec <- xnam
                       if (!is.null(rn)){ 
                         xord <- cbind(xord, rn)
                         namvec <- c(namvec, "rn")
                         labcol <- "rn"
                       }
                       colnames(xord) <- namvec
                  }
           else{
           ## use length n specs in plotting
             if (!is.null(sizecol)) 
               stop("a numeric vector-valued x cannot be combined with separate sizecol")
             sizecol <- xnam
             nc <- 1
             namvec <- xnam
             xord <- as.data.frame(as.matrix(xord))
                ## using as.matrix first 
                ## prevents the names from becoming 
                ## a column of the data frame 
                ## (they are row names instead)
             if (length(groupcol)==n) {
                hilf <- deparse(substitute(groupcol))
                if (hilf %in% namvec){
                  if (is.numeric(xord[[hilf]]))
                    hilf <- paste(hilf,"factor", sep=".")
                }
                if (!hilf %in% namvec){
                  namvec <- c(namvec, hilf)
                  xord <- cbind(xord, as.factor(groupcol))
                  nc <- nc+1
                }
                groupcol <- hilf
              }else 
                if (!is.null(groupcol)) warning("invalid groupcol ignored")
           if (length(labcol)==n) {
             hilf <- deparse(substitute(labcol))
             if (!hilf %in% namvec){
                namvec <- c(namvec, deparse(substitute(labcol)))
                xord <- cbind(xord, labcol)
                nc <- nc+1
             }
             labcol <- hilf
           }else{
             if (!is.null(labcol)) warning("invalid labcol ignored")
            if (is.null(labcol)) 
              if (!is.null(rn)){
                if ("rn" %in% namvec) 
                  warning("variable name rn conflicts with default column name for labels")
               else{
                 namvec <- c(namvec, "rn")
                 xcord <- cbind(xord, rn)
                 nc <- nc+1
                 labcol <- "rn"
               }
             }
           } 
             
        colnames(xord) <- namvec
       }
    } 
  }
  else {
    ## x is matrix or data frame
    ## check column selections
    nc <- ncol(x)
    n <- nrow(x)
    rn <- rownames(x)   ## vector or NULL
    namvec <- colnames(x)
    if (is.numeric(sizecol)){
      stopifnot(sizecol %in% 1:nc)
      sizecol <- namvec[sizecol]
    }
    if (is.numeric(groupcol)) {
      stopifnot(groupcol %in% 1:nc)
      groupcol <- namvec[groupcol]
    }
    if (is.numeric(labcol)) {
      stopifnot(labcol %in% 1:nc)
      labcol <- namvec[labcol]
    }
    if (is.character(sizecol)) stopifnot(sizecol %in% namvec)
    if (is.character(groupcol)) stopifnot(groupcol %in% namvec)
    if (is.character(labcol)) stopifnot(labcol %in% namvec)
    if (is.null(sizecol)) {
      x <- cbind(x, constant=rep(1, nrow(x)))
      sizecol <- "constant"
      namvec <- c(namvec, "constant")
    }
    if (is.null(labcol) && !is.null(rn) && !("rn" %in% namvec)){
      namvec <- c(namvec, "rn")
      x <- cbind(x, rn)
      colnames(x) <- namvec
      labcol <- "rn"
    }
    xord <- x
   } 
    ## now xord is a data frame or matrix with column names
    ## col specifications are NULL or valid column names for xord
    ## and sizecol is not NULL

    ## groupsort 
    go <- 1:nrow(xord)
    x <- xord   ### before potential resorting
    if (!is.null(groupcol)){
        if (groupsort){
           go <- order(x[,groupcol], decreasing = decreasing)
           ## revert color palettes for decreasing order
           if (decreasing) {
             pal <- rev(pal)
             col.line <- rev(col.line)
             col.lab <- rev(col.lab)
           }
        }
        xord <- x[go,]
        grp <- xord[,groupcol]
    } else 
    grp <- rep(1, nrow(x))
    if (!is.factor(grp)) grp <- as.factor(grp)
    gnum <- as.numeric(grp)  
    ## grp is the group index factor
    ## gnum is used for picking colors
    
    if (!is.null(labcol))
      labs <- xord[,labcol]
    else labs <- NULL

  ## handle colors; areColors is TRUE for all colors, zero length vectors and NA
  if (is.null(col.lab)){
    farbe1 <- t(col2rgb(rep(col.lab.defaults[1], length(pal))))
    farbe2 <- t(col2rgb(rep(col.lab.defaults[2], length(pal))))
    ref <- t(col2rgb(pal))
    ## logical vector for picking suitable default color
    pick1 <- rowSums((farbe1-ref)^2) > rowSums((farbe2-ref)^2)
    col.lab <- rep(col.lab.defaults[2], length(pal))
    col.lab[pick1] <- col.lab.defaults[1]
  }
  if (length(col.lab) > 1 && !length(col.lab)==length(pal))
    stop("col.lab must have length 1 or length(pal)")
  if (is.null(col.line)) col.line <- pal
  if (length(col.line) > 1 && !length(col.line)==length(pal))
    stop("col.line must have length 1 or length(pal)")
  ## recycle colors where necessary
  if (length(pal) < length(levels(grp))){ 
      pal <- rep(pal, length(levels(grp))%/%length(pal)+1)
      if (length(col.lab)>1) col.lab <- rep(col.lab, 
                                            length(levels(grp))%/%length(col.lab)+1)
      if (length(col.line)>1) col.line <- rep(col.line, 
                                            length(levels(grp))%/%length(col.line)+1)
      warning("colors have been recycled")
  }
  if (length(cex.lab) > 1 && !length(cex.lab)==nrow(xord))
    stop("cex.lab must have length 1 or nrow(x)")
  
  aus <- circleProgressiveLayout(xord[,sizecol], sizetype = sizetype[1]) 
  ## aus contains circle center coordinates and radii
  
  ## append variables used to output data set
  aus <- cbind(aus, xord)
  if (groupsort) aus <- cbind(aus, orig.order=(1:nrow(aus))[go])
  if (plot){
    ## prepare empty chart
    if (annot && !add){
        ## if necessary prepare default annotation
        if (is.null(main)) main <- paste("Packed circle chart for", xnam)
        if (is.null(sub)){ sub <- paste("circle size:", sizecol)
        if (!is.null(groupcol) || !is.null(labcol)) 
            sub <- paste(sub, ",   ",
               ifelse(is.null(groupcol), "", paste("fill color:", groupcol)),
               ifelse(!(is.null(groupcol) || is.null(labcol)), ",   ", ""),
               ifelse(is.null(labcol), "", paste("label:", labcol)), sep="")
        }
    }
    else {
        sub <- NULL
        main <- NULL
        if (!add) opar <- par(mar=c(0,0,0,0))
    }
    xlims <- range(aus$x-aus$radius, aus$x + aus$radius, na.rm = TRUE)
    ylims <- range(aus$y-aus$radius, aus$y + aus$radius, na.rm = TRUE)

print(main)
    if (!add)
    plot(0, 0, axes=FALSE, xlim=xlims, ylim=ylims, main=main, 
         type="n", asp=1, xlab="", ylab="", sub=sub,...)
  
    lbcs <- col.lab
        if (length(col.lab) > 1) lbcs <- col.lab[gnum]
    fgs <- col.line
        if (length(col.line) > 1) fgs <- col.line[gnum]
    symbols(aus$x, aus$y, circles=aus$radius, inches=FALSE, 
              add=TRUE, fg=fgs, bg=pal[gnum], lwd=lwd, ...)
    
    if (!is.null(labs)) {
      if (!length(cex.lab)==length(labs)) 
        cex.lab <- pmin(0.85*2*aus$radius/strwidth(labs, cex=cex.lab)*cex.lab, cex.lab)
        text(aus$x, aus$y, labs, col=lbcs, cex=cex.lab, ...)
    }
  }  ## end of if(plot)
  if (!annot && !add) par(opar)
  invisible(aus)
}

